__all__ = [
    "__version__",
    "crc",
    "p01",
    "p02",
    "p04",
    "p05",
    "p07",
]

from e2e import crc as crc
from e2e import p01 as p01
from e2e import p02 as p02
from e2e import p04 as p04
from e2e import p05 as p05
from e2e import p07 as p07
from e2e._version import __version__



import e2e


def test_e2e_p07_protect():
    # short example
    data = bytearray(b"\x00" * 24)
    length = len(data)
    data_id = 0x0a0b0c0d

    e2e.p07.e2e_p07_protect(data, length, data_id, increment_counter=False)
    assert data == bytearray(
        b"\x1f\xb2\xe7\x37\xfc\xed\xbc\xd9"
        b"\x00\x00\x00\x18\x00\x00\x00\x00"
        b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00"
    ), data.hex(sep=" ")

    e2e.p07.e2e_p07_protect(data, length, data_id, increment_counter=True)
    assert data == bytearray(
        b"\x7b\xde\x72\x68\xb8\xe9\xbc\x27"
        b"\x00\x00\x00\x18\x00\x00\x00\x01"
        b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00"
    ), data.hex(sep=" ")

    # long example (e.g. SOME/IP)
    data = bytearray(b"\x00" * 32)
    length = len(data)
    data_id = 0x0a0b0c0d
    offset = 8  # bytes

    e2e.p07.e2e_p07_protect(
        data, length, data_id, offset=offset, increment_counter=False
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x17\xf7\xc8\x17\x32\x38\x65\xa8"
        b"\x00\x00\x00\x20\x00\x00\x00\x00"
        b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00"
    ), data.hex(sep=" ")

    e2e.p07.e2e_p07_protect(
        data, length, data_id, offset=offset, increment_counter=True
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x73\x9b\x5d\x48\x76\x3c\x65\x56"
        b"\x00\x00\x00\x20\x00\x00\x00\x01"
        b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00"
    ), data.hex(sep=" ")


def test_e2e_p07_check():
    assert (
        e2e.p07.e2e_p07_check(
            b"\x1f\xb2\xe7\x37\xfc\xed\xbc\xd9"
            b"\x00\x00\x00\x18\x00\x00\x00\x00"
            b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00",
            24,
            0x0a0b0c0d,
        )
        is True
    )
    assert (
        e2e.p07.e2e_p07_check(
            b"\x1f\xb2\xe7\x37\xfc\xed\xbc\xd9"
            b"\x00\x00\x00\x18\x00\x00\x00\x00"
            b"\x0a\x0b\x0c\x0d\x00\x00\x00\x01",
            24,
            0x0a0b0c0d,
        )
        is False
    )

    assert (
        e2e.p07.e2e_p07_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00"
            b"\x17\xf7\xc8\x17\x32\x38\x65\xa8"
            b"\x00\x00\x00\x20\x00\x00\x00\x00"
            b"\x0a\x0b\x0c\x0d\x00\x00\x00\x00",
            32,
            0x0a0b0c0d,
            offset=8,
        )
        is True
    )
    assert (
        e2e.p07.e2e_p07_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00"
            b"\x17\xf7\xc8\x17\x32\x38\x65\xa8"
            b"\x00\x00\x00\x20\x00\x00\x00\x00"
            b"\x0a\x0b\x0c\x0d\x00\x00\x00\x01",
            32,
            0x0a0b0c0d,
            offset=8,
        )
        is False
    )



import typing

def calculate_crc8(
    data: bytes, start_value: int = 0xFF, first_call: bool = True
) -> int: ...
def calculate_crc8_h2f(
    data: bytes, start_value: int = 0xFF, first_call: bool = True
) -> int: ...
def calculate_crc16(
    data: bytes, start_value: int = 0xFFFF, first_call: bool = True
) -> int: ...
def calculate_crc16_arc(
    data: bytes, start_value: int = 0x0000, first_call: bool = True
) -> int: ...
def calculate_crc32(
    data: bytes, start_value: int = 0xFFFFFFFF, first_call: bool = True
) -> int: ...
def calculate_crc32_p4(
    data: bytes, start_value: int = 0xFFFFFFFF, first_call: bool = True
) -> int: ...
def calculate_crc64(
    data: bytes, start_value: int = 0xFFFFFFFFFFFFFFFF, first_call: bool = True
) -> int: ...

CRC8_INITIAL_VALUE: typing.Final[int]
CRC8_XOR_VALUE: typing.Final[int]
CRC8_CHECK: typing.Final[int]
CRC8_MAGIC_CHECK: typing.Final[int]

CRC8H2F_INITIAL_VALUE: typing.Final[int]
CRC8H2F_XOR_VALUE: typing.Final[int]
CRC8H2F_CHECK: typing.Final[int]
CRC8H2F_MAGIC_CHECK: typing.Final[int]

CRC16_INITIAL_VALUE: typing.Final[int]
CRC16_XOR_VALUE: typing.Final[int]
CRC16_CHECK: typing.Final[int]
CRC16_MAGIC_CHECK: typing.Final[int]

CRC16ARC_INITIAL_VALUE: typing.Final[int]
CRC16ARC_XOR_VALUE: typing.Final[int]
CRC16ARC_CHECK: typing.Final[int]
CRC16ARC_MAGIC_CHECK: typing.Final[int]

CRC32_INITIAL_VALUE: typing.Final[int]
CRC32_XOR_VALUE: typing.Final[int]
CRC32_CHECK: typing.Final[int]
CRC32_MAGIC_CHECK: typing.Final[int]

CRC32P4_INITIAL_VALUE: typing.Final[int]
CRC32P4_XOR_VALUE: typing.Final[int]
CRC32P4_CHECK: typing.Final[int]
CRC32P4_MAGIC_CHECK: typing.Final[int]

CRC64_INITIAL_VALUE: typing.Final[int]
CRC64_XOR_VALUE: typing.Final[int]
CRC64_CHECK: typing.Final[int]
CRC64_MAGIC_CHECK: typing.Final[int]



def e2e_p02_protect(
    data: bytearray, length: int, data_id_list: bytes, *, increment_counter: bool = True
) -> None: ...
def e2e_p02_check(data: bytes, length: int, data_id_list: bytes) -> bool: ...



import e2e.crc


# fmt: off
def test_calculate_crc8():
    # test check
    assert e2e.crc.CRC8_CHECK == e2e.crc.calculate_crc8(b"123456789")

    # test magic check
    assert 0x59 == e2e.crc.calculate_crc8(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC8_MAGIC_CHECK 
        == e2e.crc.calculate_crc8(b"\x00\x00\x00\x00\x59") ^ e2e.crc.CRC8_XOR_VALUE
    )

    # test different input types
    assert 0x59 == e2e.crc.calculate_crc8(bytearray(b"\x00\x00\x00\x00"))
    assert 0x59 == e2e.crc.calculate_crc8(b"\x00\x00\x00\x00")

    assert 0x37 == e2e.crc.calculate_crc8(b"\xF2\x01\x83")
    assert 0x79 == e2e.crc.calculate_crc8(b"\x0F\xAA\x00\x55")
    assert 0xB8 == e2e.crc.calculate_crc8(b"\x00\xFF\x55\x11")
    assert 0xCB == e2e.crc.calculate_crc8(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0x8C == e2e.crc.calculate_crc8(b"\x92\x6B\x55")
    assert 0x74 == e2e.crc.calculate_crc8(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC8_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc8(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0xCB == crc


def test_calculate_crc8_h2f():
    # test check
    assert e2e.crc.CRC8H2F_CHECK == e2e.crc.calculate_crc8_h2f(b"123456789")

    # test magic check
    assert 0x12 == e2e.crc.calculate_crc8_h2f(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC8H2F_MAGIC_CHECK 
        == e2e.crc.calculate_crc8_h2f(b"\x00\x00\x00\x00\x12") ^ e2e.crc.CRC8H2F_XOR_VALUE
    )

    # test different input types
    assert 0x12 == e2e.crc.calculate_crc8_h2f(bytearray(b"\x00\x00\x00\x00"))
    assert 0x12 == e2e.crc.calculate_crc8_h2f(b"\x00\x00\x00\x00")

    assert 0xC2 == e2e.crc.calculate_crc8_h2f(b"\xF2\x01\x83")
    assert 0xC6 == e2e.crc.calculate_crc8_h2f(b"\x0F\xAA\x00\x55")
    assert 0x77 == e2e.crc.calculate_crc8_h2f(b"\x00\xFF\x55\x11")
    assert 0x11 == e2e.crc.calculate_crc8_h2f(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0x33 == e2e.crc.calculate_crc8_h2f(b"\x92\x6B\x55")
    assert 0x6C == e2e.crc.calculate_crc8_h2f(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC8H2F_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc8_h2f(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0x11 == crc


def test_calculate_crc16():
    # test check
    assert e2e.crc.CRC16_CHECK == e2e.crc.calculate_crc16(b"123456789")

    # test magic check
    assert 0x84C0 == e2e.crc.calculate_crc16(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC16_MAGIC_CHECK 
        == e2e.crc.calculate_crc16(b"\x00\x00\x00\x00\x84\xC0") ^ e2e.crc.CRC16_XOR_VALUE
    )

    # test different input types
    assert 0x84C0 == e2e.crc.calculate_crc16(bytearray(b"\x00\x00\x00\x00"))
    assert 0x84C0 == e2e.crc.calculate_crc16(b"\x00\x00\x00\x00")

    assert 0xD374 == e2e.crc.calculate_crc16(b"\xF2\x01\x83")
    assert 0x2023 == e2e.crc.calculate_crc16(b"\x0F\xAA\x00\x55")
    assert 0xB8F9 == e2e.crc.calculate_crc16(b"\x00\xFF\x55\x11")
    assert 0xF53F == e2e.crc.calculate_crc16(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0x0745 == e2e.crc.calculate_crc16(b"\x92\x6B\x55")
    assert 0x1D0F == e2e.crc.calculate_crc16(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC16_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc16(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0xF53F == crc


def test_calculate_crc16_arc():
    # test check
    assert e2e.crc.CRC16ARC_CHECK == e2e.crc.calculate_crc16_arc(b"123456789")

    # test magic check
    assert 0x0000 == e2e.crc.calculate_crc16_arc(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC16ARC_MAGIC_CHECK 
        == e2e.crc.calculate_crc16_arc(b"\x00\x00\x00\x00\x00\x00") ^ e2e.crc.CRC16ARC_XOR_VALUE
    )

    # test different input types
    assert 0x0000 == e2e.crc.calculate_crc16_arc(bytearray(b"\x00\x00\x00\x00"))
    assert 0x0000 == e2e.crc.calculate_crc16_arc(b"\x00\x00\x00\x00")

    assert 0xC2E1 == e2e.crc.calculate_crc16_arc(b"\xF2\x01\x83")
    assert 0x0BE3 == e2e.crc.calculate_crc16_arc(b"\x0F\xAA\x00\x55")
    assert 0x6CCF == e2e.crc.calculate_crc16_arc(b"\x00\xFF\x55\x11")
    assert 0xAE98 == e2e.crc.calculate_crc16_arc(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0xE24E == e2e.crc.calculate_crc16_arc(b"\x92\x6B\x55")
    assert 0x9401 == e2e.crc.calculate_crc16_arc(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC16ARC_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc16_arc(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0xAE98 == crc


def test_calculate_crc32():
    # test check
    assert e2e.crc.CRC32_CHECK == e2e.crc.calculate_crc32(b"123456789")

    # test magic check
    assert 0x2144DF1C == e2e.crc.calculate_crc32(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC32_MAGIC_CHECK 
        == e2e.crc.calculate_crc32(b"\x00\x00\x00\x00\x1C\xDF\x44\x21") ^ e2e.crc.CRC32_XOR_VALUE
    )

    # test different input types
    assert 0x2144DF1C == e2e.crc.calculate_crc32(bytearray(b"\x00\x00\x00\x00"))
    assert 0x2144DF1C == e2e.crc.calculate_crc32(b"\x00\x00\x00\x00")

    assert 0x24AB9D77 == e2e.crc.calculate_crc32(b"\xF2\x01\x83")
    assert 0xB6C9B287 == e2e.crc.calculate_crc32(b"\x0F\xAA\x00\x55")
    assert 0x32A06212 == e2e.crc.calculate_crc32(b"\x00\xFF\x55\x11")
    assert 0xB0AE863D == e2e.crc.calculate_crc32(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0x9CDEA29B == e2e.crc.calculate_crc32(b"\x92\x6B\x55")
    assert 0xFFFFFFFF == e2e.crc.calculate_crc32(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC32_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc32(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0xB0AE863D == crc


def test_calculate_crc32_p4():
    # test check
    assert e2e.crc.CRC32P4_CHECK == e2e.crc.calculate_crc32_p4(b"123456789")

    # test magic check
    assert 0x6FB32240 == e2e.crc.calculate_crc32_p4(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC32P4_MAGIC_CHECK 
        == (
            e2e.crc.calculate_crc32_p4(b"\x00\x00\x00\x00\x40\x22\xB3\x6F") 
            ^ e2e.crc.CRC32P4_XOR_VALUE
        )
    )

    # test different input types
    assert 0x6FB32240 == e2e.crc.calculate_crc32_p4(bytearray(b"\x00\x00\x00\x00"))
    assert 0x6FB32240 == e2e.crc.calculate_crc32_p4(b"\x00\x00\x00\x00")

    assert 0x4F721A25 == e2e.crc.calculate_crc32_p4(b"\xF2\x01\x83")
    assert 0x20662DF8 == e2e.crc.calculate_crc32_p4(b"\x0F\xAA\x00\x55")
    assert 0x9BD7996E == e2e.crc.calculate_crc32_p4(b"\x00\xFF\x55\x11")
    assert 0xA65A343D == e2e.crc.calculate_crc32_p4(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0xEE688A78 == e2e.crc.calculate_crc32_p4(b"\x92\x6B\x55")
    assert 0xFFFFFFFF == e2e.crc.calculate_crc32_p4(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC32P4_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc32_p4(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0xA65A343D == crc


def test_calculate_crc64():
    # test check
    assert e2e.crc.CRC64_CHECK == e2e.crc.calculate_crc64(b"123456789")

    # test magic check
    assert 0xF4A586351E1B9F4B == e2e.crc.calculate_crc64(b"\x00\x00\x00\x00")
    assert (
        e2e.crc.CRC64_MAGIC_CHECK 
        == (
            e2e.crc.calculate_crc64(b"\x00\x00\x00\x00\x4B\x9F\x1B\x1E\x35\x86\xA5\xF4") 
            ^ e2e.crc.CRC64_XOR_VALUE
        )
    )

    # test different input types
    assert 0xF4A586351E1B9F4B == e2e.crc.calculate_crc64(bytearray(b"\x00\x00\x00\x00"))
    assert 0xF4A586351E1B9F4B == e2e.crc.calculate_crc64(b"\x00\x00\x00\x00")

    assert 0x319C27668164F1C6 == e2e.crc.calculate_crc64(b"\xF2\x01\x83")
    assert 0x54C5D0F7667C1575 == e2e.crc.calculate_crc64(b"\x0F\xAA\x00\x55")
    assert 0xA63822BE7E0704E6 == e2e.crc.calculate_crc64(b"\x00\xFF\x55\x11")
    assert 0x701ECEB219A8E5D5 == e2e.crc.calculate_crc64(b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF")
    assert 0x5FAA96A9B59F3E4E == e2e.crc.calculate_crc64(b"\x92\x6B\x55")
    assert 0xFFFFFFFF00000000 == e2e.crc.calculate_crc64(b"\xFF\xFF\xFF\xFF")

    # test start_value and first_call
    first_call = True
    crc = e2e.crc.CRC64_INITIAL_VALUE
    data = b"\x33\x22\x55\xAA\xBB\xCC\xDD\xEE\xFF"
    for i, _val in enumerate(data):
        crc = e2e.crc.calculate_crc64(data[i:i+1], start_value=crc, first_call=first_call)
        first_call = False
    assert 0x701ECEB219A8E5D5 == crc
# fmt: on



import e2e


def test_e2e_p02_protect():
    data = bytearray(b"\x00" * 8)
    length = len(data) - 1
    data_id_list = b"\x00" * 16

    e2e.p02.e2e_p02_protect(data, length, data_id_list)
    assert b"\x45\x01\x00\x00\x00\x00\x00\x00" == bytes(data)

    data = bytearray(range(8))
    length = len(data) - 1
    data_id_list = bytes(range(16))
    e2e.p02.e2e_p02_protect(data, length, data_id_list)
    assert b"\xbc\x02\x02\x03\x04\x05\x06\x07" == bytes(data)

    data = bytearray(range(8))
    length = len(data) - 1
    data_id_list = bytes(range(16))
    e2e.p02.e2e_p02_protect(data, length, data_id_list, increment_counter=False)
    assert b"\x61\x01\x02\x03\x04\x05\x06\x07" == bytes(data)


def test_e2e_p02_check():
    data = bytearray(range(8))
    data[1] = 2
    length = len(data) - 1
    data_id_list = bytes(range(16))
    assert (
        e2e.p02.e2e_p02_check(b"\xbc\x02\x02\x03\x04\x05\x06\x07", length, data_id_list)
        is True
    )
    assert (
        e2e.p02.e2e_p02_check(b"\xbc\x01\x02\x03\x04\x05\x06\x07", length, data_id_list)
        is False
    )



import e2e


def test_e2e_p04_protect():
    # short example
    data = bytearray(b"\x00" * 16)
    length = len(data)
    data_id = 0x0a0b0c0d

    e2e.p04.e2e_p04_protect(data, length, data_id, increment_counter=False)
    assert data == bytearray(
        b"\x00\x10\x00\x00\x0a\x0b\x0c\x0d"
        b"\x86\x2b\x05\x56\x00\x00\x00\x00"
    ), data.hex(sep=" ")

    e2e.p04.e2e_p04_protect(data, length, data_id, increment_counter=True)
    assert data == bytearray(
        b"\x00\x10\x00\x01\x0a\x0b\x0c\x0d"
        b"\xa5\x8e\x68\x07\x00\x00\x00\x00"
    ), data.hex(sep=" ")

    # long example (e.g. SOME/IP)
    data = bytearray(b"\x00" * 24)
    length = len(data)
    data_id = 0x0a0b0c0d
    offset = 8  # bytes

    e2e.p04.e2e_p04_protect(
        data, length, data_id, offset=offset, increment_counter=False
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x00\x18\x00\x00\x0a\x0b\x0c\x0d"
        b"\x69\xd7\x50\x2e\x00\x00\x00\x00"
    ), data.hex(" ")

    e2e.p04.e2e_p04_protect(
        data, length, data_id, offset=offset, increment_counter=True
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00"
        b"\x00\x18\x00\x01\x0a\x0b\x0c\x0d"
        b"\x4a\x72\x3d\x7f\x00\x00\x00\x00"
    ), data.hex(" ")


def test_e2e_p04_check():
    assert (
        e2e.p04.e2e_p04_check(
            b"\x00\x10\x00\x00\x0a\x0b\x0c\x0d"
            b"\x86\x2b\x05\x56\x00\x00\x00\x00",
            16,
            0x0a0b0c0d,
        )
        is True
    )
    assert (
        e2e.p04.e2e_p04_check(
            b"\x00\x10\x00\x01\x0a\x0b\x0c\x0d"
            b"\x86\x2b\x05\x56\x00\x00\x00\x00",
            16,
            0x0a0b0c0d,
        )
        is False
    )

    assert (
        e2e.p04.e2e_p04_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00"
            b"\x00\x18\x00\x00\x0a\x0b\x0c\x0d"
            b"\x69\xd7\x50\x2e\x00\x00\x00\x00",
            24,
            0x0a0b0c0d,
            offset=8,
        )
        is True
    )
    assert (
        e2e.p04.e2e_p04_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00"
            b"\x00\x18\x00\x01\x0a\x0b\x0c\x0d"
            b"\x69\xd7\x50\x2e\x00\x00\x00\x00",
            24,
            0x0a0b0c0d,
            offset=8,
        )
        is False
    )



import typing

E2E_P01_DATAID_BOTH: typing.Final[int]
E2E_P01_DATAID_ALT: typing.Final[int]
E2E_P01_DATAID_LOW: typing.Final[int]
E2E_P01_DATAID_NIBBLE: typing.Final[int]

def e2e_p01_protect(
    data: bytearray,
    length: int,
    data_id: int,
    *,
    data_id_mode: int = E2E_P01_DATAID_BOTH,
    increment_counter: bool = True,
) -> None: ...
def e2e_p01_check(
    data: bytearray,
    length: int,
    data_id: int,
    *,
    data_id_mode: int = E2E_P01_DATAID_BOTH,
) -> bool: ...



def e2e_p07_protect(
    data: bytearray,
    length: int,
    data_id: int,
    *,
    offset: int = 0,
    increment_counter: bool = True,
) -> None: ...
def e2e_p07_check(
    data: bytes, length: int, data_id: int, *, offset: int = 0
) -> bool: ...



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Imports -----------------------------------------------------------------
import e2e

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "autosar-e2e"
copyright = "2022, Artur Drogunow"
author = "Artur Drogunow"

version = e2e.__version__.split("-", maxsplit=1)[0]
release = e2e.__version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
]

intersphinx_mapping = {"python": ("https://docs.python.org/3/", None)}

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "furo"
html_static_path = []



import e2e


def test_e2e_p05_protect():
    # short example
    data = bytearray(b"\x00" * 8)
    length = len(data) - 2
    data_id = 0x1234

    e2e.p05.e2e_p05_protect(data, length, data_id, increment_counter=False)
    assert data == bytearray(b"\x1C\xCA\x00\x00\x00\x00\x00\x00"), data

    e2e.p05.e2e_p05_protect(data, length, data_id, increment_counter=True)
    assert data == bytearray(b"\xCF\x8D\x01\x00\x00\x00\x00\x00"), data

    # long example (e.g. SOME/IP)
    data = bytearray(b"\x00" * 16)
    length = len(data) - 2
    data_id = 0x1234
    offset = 8  # bytes

    e2e.p05.e2e_p05_protect(
        data, length, data_id, offset=offset, increment_counter=False
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00\x28\x91\x00\x00\x00\x00\x00\x00"
    ), data

    e2e.p05.e2e_p05_protect(
        data, length, data_id, offset=offset, increment_counter=True
    )
    assert data == bytearray(
        b"\x00\x00\x00\x00\x00\x00\x00\x00\xFB\xD6\x01\x00\x00\x00\x00\x00"
    ), data


def test_e2e_p05_check():
    assert (
        e2e.p05.e2e_p05_check(
            b"\x1C\xCA\x00\x00\x00\x00\x00\x00",
            6,
            0x1234,
        )
        is True
    )
    assert (
        e2e.p05.e2e_p05_check(
            b"\x1C\xCA\x01\x00\x00\x00\x00\x00",
            6,
            0x1234,
        )
        is False
    )

    assert (
        e2e.p05.e2e_p05_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00\x28\x91\x00\x00\x00\x00\x00\x00",
            14,
            0x1234,
            offset=8,
        )
        is True
    )
    assert (
        e2e.p05.e2e_p05_check(
            b"\x00\x00\x00\x00\x00\x00\x00\x00\x28\x91\x01\x00\x00\x00\x00\x00",
            14,
            0x1234,
            offset=8,
        )
        is False
    )



import e2e


def test_e2e_p01_protect_both():
    data = bytearray(b"\x00" * 8)

    # do not increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_BOTH,
        increment_counter=False,
    )
    assert data == bytearray(b"\xCC\x00\x00\x00\x00\x00\x00\x00"), data

    # increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_BOTH,
        increment_counter=True,
    )
    assert data == bytearray(b"\x91\x01\x00\x00\x00\x00\x00\x00"), data


def test_e2e_p01_protect_alt():
    data = bytearray(b"\x00" * 8)

    # do not increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_ALT,
        increment_counter=False,
    )
    assert data == bytearray(b"\xCE\x00\x00\x00\x00\x00\x00\x00"), data

    # increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_ALT,
        increment_counter=True,
    )
    assert data == bytearray(b"\x02\x01\x00\x00\x00\x00\x00\x00"), data


def test_e2e_p01_protect_low():
    data = bytearray(b"\x00" * 8)

    # do not increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_LOW,
        increment_counter=False,
    )
    assert data == bytearray(b"\xCE\x00\x00\x00\x00\x00\x00\x00"), data

    # increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_LOW,
        increment_counter=True,
    )
    assert data == bytearray(b"\x93\x01\x00\x00\x00\x00\x00\x00"), data


def test_e2e_p01_protect_nibble():
    data = bytearray(b"\x00" * 8)

    # do not increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_NIBBLE,
        increment_counter=False,
    )
    assert data == bytearray(b"\x2A\x10\x00\x00\x00\x00\x00\x00"), data

    # increment counter
    e2e.p01.e2e_p01_protect(
        data=data,
        data_id=0x123,
        length=7,
        data_id_mode=e2e.p01.E2E_P01_DATAID_NIBBLE,
        increment_counter=True,
    )
    assert data == bytearray(b"\x77\x11\x00\x00\x00\x00\x00\x00"), data


def test_e2e_p01_check_both():
    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\xCC\x00\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_BOTH,
        )
        is True
    )

    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\xCC\x10\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_BOTH,
        )
        is False
    )


def test_e2e_p01_check_alt():
    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\xCE\x00\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_ALT,
        )
        is True
    )

    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\xCE\x10\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_ALT,
        )
        is False
    )


def test_e2e_p01_check_low():
    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\xCE\x00\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_LOW,
        )
        is True
    )

    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\x93\x11\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_LOW,
        )
        is False
    )


def test_e2e_p01_check_nibble():
    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\x2A\x10\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_NIBBLE,
        )
        is True
    )

    assert (
        e2e.p01.e2e_p01_check(
            data=bytearray(b"\x77\x21\x00\x00\x00\x00\x00\x00"),
            data_id=0x123,
            length=7,
            data_id_mode=e2e.p01.E2E_P01_DATAID_NIBBLE,
        )
        is False
    )
